% calcRotatedCovariance : Calculate OAGSM covariances, by patch rotation
%
% [C,theta_zero_patches] = calcRotatedCovariance(fullband,localori,rotmask,theta_list,K,Uf)
%
% Calculate rotated covariance matrices by upsampling the pyramid bands in
% fullband by 2^Uf in the frequency domain, rotating and steering patches by
% localori+theta for theta sampling specified in theta_list (using linear
% interpolation on upsampled bands), and computing sample outer products
% (using patches from locations specified by rotmask).
%
% Inputs:
% "fullband" - the pyramid band represented in fullband data structure
% "localori" - array of angles giving local orientation, size should be
% consistent with fullband
% "rotmask" - logical array indicating which patch locations should be
% used in covariance calculation
% "theta_list" - 1 x Ntheta list of angle samplings between -pi and pi
% "K" - generalized patch descriptor (see oagsm_data_structures)
% "Uf" - upsampling factor, bands will be upsampled by 2^Uf in fourier domain
%
% Outputs:
% "C" - 1xNtheta cell array of covariance matrices, C{ktheta} is formed
% from rotating patches by theta_list(k) beyond their dominant orientation.
% These are calculated as 'joint covariances', treating all orientation
% bands as part of the generalized neighborhood, covariances for single
% orientation bands may be extracted by using getBlockDiagBand.
% "theta_zero_patches" - set of patches rotated to their dominant
% orientation, returned if theta_list includes 0

function [C,theta_zero_patches] = calcRotatedCovariance(fullband,localori,rotmask,theta_list,K,Uf)
nbands=K.nbands;
Npatches = sum(rotmask(:));
Ntheta=numel(theta_list);
fprintf('Calculating Cori - (Ntheta=%g) ',Ntheta);  
small_size = [size(fullband.child,1),size(fullband.child,2)]; 
% size of parent and child should be the same

% separate parent,child code into blocks
% always run child code,only run parent code if using parent

upfactor_c = 2^Uf; % parent has already been interpolated up one octave
boundary_c = max(2*K.child+1);
pfactor_c=1;   
big_size_c = upfactor_c * small_size;
big_boundary_c = boundary_c * upfactor_c;
[X_big_c,Y_big_c] = extendedmeshgrid(big_size_c,big_boundary_c);
patches.child = zeros(2*K.child(1)+1,2*K.child(2)+1,Npatches,nbands);

if (K.useparent)
  upfactor_p = 2^(Uf-1); % in fourier domain, so should need one octave less here  
  boundary_p = max(2*K.parent+1);
  pfactor_p=K.pfactor; 
  big_size_p = upfactor_p * small_size;
  big_boundary_p = boundary_p * upfactor_p;
  [X_big_p,Y_big_p] = extendedmeshgrid(big_size_p,big_boundary_p);
  patches.parent = zeros(2*K.parent(1)+1,2*K.parent(2)+1,Npatches,nbands);
end

[XI,YI] = meshgrid(1:small_size(2),1:small_size(1));  
% interpolate up all of the bands at once 
% fprintf('upsampling bands - ');
for b=1:nbands
  big_child_e{b}=extendBoundary(upsampleFourier(fullband.child(:,:,b),big_size_c),...
                                 big_boundary_c,'method','circular');
  if (K.useparent)
    big_parent_e{b}=extendBoundary(upsampleFourier(fullband.parent(:,:,b),big_size_p),...
                                    big_boundary_p,'method','circular');
  end
end
% now loop over theta, rotate patches, transform patches, compute covariance
% fprintf('computing covariances - ') 
for ktheta=1:Ntheta
  loriplustheta=localori(:)+theta_list(ktheta);
  fprintf('%g ',ktheta);
  for b = 1:nbands 
    patches.child(:,:,:,b) = scalarRotate(X_big_c,Y_big_c,big_child_e{b},XI,YI,K.child,loriplustheta,...
                                           rotmask,upfactor_c,pfactor_c);
    if (K.useparent)
      patches.parent(:,:,:,b)= scalarRotate(X_big_p,Y_big_p,big_parent_e{b},XI,YI,K.parent,loriplustheta,...
                                             rotmask,upfactor_p,pfactor_p);
    end
  end
  % transform patches by steering
  steermat = calcSteeringMatrices(nbands,loriplustheta(rotmask(:)));
  tpatches.child = steerpatches(patches.child,steermat);
  if (K.useparent)
    tpatches.parent = steerpatches(patches.parent,steermat);
  end  
  
  % save tpatches if angle is zero
  if(theta_list(ktheta)==0)
    theta_zero_patches=tpatches;
  end
  
  % compute covariance
  y = patchvec(tpatches,K);
  C{ktheta} = y*y'/size(y,2); 
end
fprintf('\n');




