% oagsmncBand : perform oagsm/noricomp denoising on one subband
%
% [fullband_d,A]=oagsmncBand(fullband,Cn,Cori,Cnor,logz,K,beta)
%
% Inputs:
% fullband - noisy subband (as fullband data structure)
% Cn - noise covariance
% Cori - cell list of oriented covariances
% Cnor - non-oriented (GSM) covariance
% logz - list of log(z) values, z being the hidden scalar multiplier
% K - generalized neighborhood descriptor
% beta - prior on oriented component (beta)
% If beta=0, then this code does GSM calculation, and Cori may be empty.
% If beta=1, then this code does OAGSM calculation, and Cnor may be empty
%
% Outputs:
% fullband_d - denoised band, containing only child band
% A - structure for passing debugging data

function [fullband_d,A]=oagsmncBand(fullband,Cn,Cori,Cnor,logz,K,beta)

% Flags do_ori and do_nor control whether to do integration over
% parameters for nor calculation (using Cnor) and/ or ori calculation
% (using Cori)
% Passing in beta = 0 will then do gsm calculation, passing beta=1
% will do only oagsm calculation. 
if beta==1
  do_ori=1;
  do_nor=0;
elseif beta==0
  do_ori=0;
  do_nor=1;
else
  do_ori=1;
  do_nor=1;
end  

Ntheta=length(Cori);
Nz = length(logz);
nbands=K.nbands;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% precalculate wiener filters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% indices of C have form
% child band 1
% child band 2
% ...
% child band nbands
% parent band 1
% ....
% parent band nbands
%
% centerind should be a list of indices indicating the center
% positions of the child bands - it is these rows of the wiener filter
% that I want, as I will only estimate the center coefficient of each
% patch
childblocksize = (2*K.child(1)+1)*(2*K.child(2)+1);
centerind = ceil(childblocksize/2) + ([0:nbands-1])*childblocksize;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate wiener filters for ori and nor covariance
% will use invYcov_ori, sqrtdetYcov_ori, wienerfilt_nor,
% invYcov_nor, sqrtdetYcov_nor and wienerfilt_ori 
% in denoising calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for kz = 1:length(logz)
  z=exp(logz(kz));
  if do_ori
    for ktheta=1:Ntheta
      % wiener filters for ori covariance
      invYcov_ori{kz,ktheta}=inv(z*Cori{ktheta}+Cn);
      sqrtdetYcov_ori(kz,ktheta)=sqrt(det(z*Cori{ktheta}+Cn));    
      % taking the determinant this way ... careful about over/underflow
      wftotal=z*Cori{ktheta}*inv(z*Cori{ktheta}+Cn);
      wienerfilt_ori{kz,ktheta}=wftotal(centerind,:);
    end
  end
  if do_nor
    % wiener filters for nor covariance
    invYcov_nor{kz}=inv(z*Cnor+Cn);
    sqrtdetYcov_nor(kz)=sqrt(det(z*Cnor+Cn));
    wftotal=z*Cnor*inv(z*Cnor+Cn);
    wienerfilt_nor{kz}=wftotal(centerind,:);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main loop - integrate over theta and logz
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

patches=packVecPatches(fullband,K);

Npatches = size(patches,2);
xhat_sum_p=zeros(nbands,Npatches);
norm = zeros(1,Npatches);

fprintf('Denoising (Nz=%g) ',Nz);
for kz=1:Nz
  fprintf(' %g',kz);
  if do_ori
    for ktheta=1:Ntheta
      % ori calculation
      wienerfilt_tmp=wienerfilt_ori{kz,ktheta};
      xhat_p=wienerfilt_tmp*patches;
      % now xhat_p is size [nbands,Npatches]
      invYcov_tmp=invYcov_ori{kz,ktheta};
      quadform_p = -(1/2)*sum(patches.*(invYcov_tmp*patches));
      % w is proportional to beta*p_ori(y|z,theta)
      w = (beta/(Nz*Ntheta))*(1/sqrtdetYcov_ori(kz,ktheta)).*exp(quadform_p);
      % w must be [1,Npatches] for next line to work properly
      w=w(:)';    
      xhat_sum_p=xhat_sum_p+w(ones(1,nbands),:).*xhat_p;
      norm=norm+w;  
    end
  end
  if do_nor
    % nor calculation
    wienerfilt_tmp=wienerfilt_nor{kz};
    xhat_p=wienerfilt_tmp*patches;
    invYcov_tmp=invYcov_nor{kz};
    quadform_p=-(1/2)*sum(patches.*(invYcov_tmp*patches));
    % w is proportional to (1-beta)*p_nor(y|z)
    w=((1-beta)/Nz)*(1/sqrtdetYcov_nor(kz)).*exp(quadform_p);
    w=w(:)';
    xhat_sum_p=xhat_sum_p+w(ones(1,nbands),:).*xhat_p;
    norm=norm+w;
  end 
end

fprintf('\n');
norm(find(norm==0))=1;

xhat=xhat_sum_p./norm(ones(1,nbands),:);
% answer should returned in form [M,N,nbands] ->
% as xhat is currently size [nbands,Npatches], the xhat'
% is necessary due to MATLAB's column-major memory ordering
fullband_d=reshape(xhat',size(fullband.child));

if any(isnan(vec(fullband_d)))
  fprintf('NaN detected in oagsmband');
end

A.norm=norm;
